subroutine NSSANM(lx1,ng,nmix,iqfr,qfr,mat,fd,L,R,savg)
!
!-----------------------------------------------------------------------
!
!Purpose:
! Compute the ANM volume fluxes and boundary fluxes and currents using
! a solution of one- and two-node relations.
!
!Copyright:
! Copyright (C) 2022 Ecole Polytechnique de Montreal
! This library is free software; you can redistribute it and/or
! modify it under the terms of the GNU Lesser General Public
! License as published by the Free Software Foundation; either
! version 2.1 of the License, or (at your option) any later version
!
!Author(s): A. Hebert
!
!Parameters: input
! lx1     number of nodes in the nodal calculation.
! ng      number of energy groups.
! nmix    number of material mixtures in the nodal calculation.
! iqfr    node-ordered physical albedo indices.
! qfr     albedo function information.
! mat     material mixture index in eacn node.
! fd      discontinuity factors
! L       left ANM coupling matrices in each node.
! R       right ANM coupling matrices in each node.
! savg    nodal fluxes.
!
!Parameters: output
! savg    boundary fluxes and currents.
!
!-----------------------------------------------------------------------
  !
  !----
  !  subroutine arguments
  !----
  integer,intent(in) :: lx1,ng,nmix,iqfr(6,lx1),mat(lx1)
  real,intent(in) :: qfr(6,lx1,ng),fd(nmix,2,ng,ng)
  complex(kind=8), dimension(ng,2*ng,lx1),intent(in) :: L,R
  real, dimension(4*lx1+1,ng),intent(inout) :: savg
  !----
  ! allocatable arrays
  !----
  real, allocatable, dimension(:,:) :: A,B,Lambda
  !----
  ! scratch storage allocation
  !----
  allocate(A(ng,ng+1),B(ng,ng),Lambda(ng,ng))
  !----
  ! compute boundary currents
  ! left one-node relation
  !----
  A(:ng,:ng+1)=0.0
  if((iqfr(1,1) > 0).or.(iqfr(1,1) == -1)) then
    ! physical albedo
    Lambda(:ng,:ng)=0.0
    do ig=1,ng
      Lambda(ig,ig)=qfr(1,1,ig)
    enddo
    A(:ng,:ng)=real(matmul(Lambda(:ng,:ng),L(:ng,ng+1:2*ng,1)),4)
    B(:ng,:ng)=real(matmul(Lambda(:ng,:ng),L(:ng,:ng,1)),4)
    do ig=1,ng
      A(ig,ig)=1.0+A(ig,ig)
    enddo
    A(:ng,ng+1)=-matmul(B(:ng,:ng),savg(1,:ng))
  else if(iqfr(1,1) == -2) then
    ! zero net current
    do ig=1,ng
      A(ig,ig)=1.0
    enddo
  else if(iqfr(1,1) == -3) then
    ! zero flux
    A(:ng,:ng)=real(L(:ng,ng+1:2*ng,1),4)
    A(:ng,ng+1)=real(-matmul(L(:ng,:ng,1),savg(1,:ng)),4)
  else
    call XABORT('NSSANM: illegal left boundary condition.')
  endif
  call ALSB(ng,1,A,ier,ng)
  if(ier /= 0) call XABORT('NSSANM: singular matrix.(1)')
  savg(3*lx1+1,:ng)=A(:ng,ng+1)
  ! two-node relations
  do i=2,lx1
    A(:ng,:ng)=real(matmul(fd(mat(i-1),2,:,:),R(:ng,ng+1:2*ng,i-1))- &
                  & matmul(fd(mat(i),1,:,:),L(:ng,ng+1:2*ng,i)),4)
    A(:ng,ng+1)=-real(matmul(matmul(fd(mat(i-1),2,:,:),R(:ng,:ng,i-1)),savg(i-1,:))- &
                  & matmul(matmul(fd(mat(i),1,:,:),L(:ng,:ng,i)),savg(i,:)),4)
    call ALSB(ng,1,A,ier,ng)
    if(ier /= 0) call XABORT('NSSANM: singular matrix.(2)')
    savg(3*lx1+i,:ng)=A(:ng,ng+1)
  enddo
  ! right one-node relation
  if((iqfr(2,lx1) > 0).or.(iqfr(2,lx1) == -1)) then
    ! physical albedo
    Lambda(:ng,:ng)=0.0
    do ig=1,ng
      Lambda(ig,ig)=qfr(2,lx1,ig)
    enddo
    A(:ng,:ng)=real(matmul(Lambda(:ng,:ng),R(:ng,ng+1:2*ng,lx1)),4)
    B(:ng,:ng)=real(matmul(Lambda(:ng,:ng),R(:ng,:ng,lx1)),4)
    do ig=1,ng
      A(ig,ig)=-1.0+A(ig,ig)
    enddo
    A(:ng,ng+1)=-matmul(B(:ng,:ng),savg(lx1,:ng))
  else if(iqfr(2,lx1) == -2) then
    ! zero net current
    do ig=1,ng
      A(2*lx1*ng+ig,2*lx1*ng+ig)=1.0
    enddo
  else if(iqfr(2,lx1) == -3) then
    ! zero flux
    A(:ng,:ng)=real(R(:ng,ng+1:2*ng,lx1),4)
    A(:ng,ng+1)=real(-matmul(R(:ng,:ng,lx1),savg(lx1,:ng)),4)
  else
    call XABORT('NSSANM: illegal right boundary condition.')
  endif
  call ALSB(ng,1,A,ier,ng)
  if(ier /= 0) call XABORT('NSSANM: singular matrix.(3)')
  savg(4*lx1+1,:ng)=A(:ng,ng+1)
  !----
  ! compute boundary fluxes
  !----
  do i=1,lx1
    savg(lx1+i,:ng)=real(matmul(L(:ng,:ng,i),savg(i,:ng))+ &
    & matmul(L(:ng,ng+1:2*ng,i),savg(3*lx1+i,:ng)),4)
    savg(2*lx1+i,:ng)=real(matmul(R(:ng,:ng,i),savg(i,:ng))+ &
    & matmul(R(:ng,ng+1:2*ng,i),savg(3*lx1+i+1,:ng)),4)
  enddo
  !----
  ! scratch storage deallocation
  !----
  deallocate(Lambda,B,A)
end subroutine NSSANM
