*DECK NSSFL3
      SUBROUTINE NSSFL3(NUN,NG,LX1,NMIX,NALB,EPSOUT,MAXOUT,MAT,XX,
     1 IQFR,QFR,DIFF,SIGR,CHI,SIGF,SCAT,BETA,FD,IMPX,KEFF,SAV)
*
*-----------------------------------------------------------------------
*
*Purpose:
* Flux calculation for the analytic nodal method in Cartesian 1D
* geometry.
*
*Copyright:
* Copyright (C) 2022 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): A. Hebert
*
*Parameters: input
* NUN     number of unknowns (=4*LX1+1).
* NG      number of energy groups.
* LX1     number of nodes in the nodal calculation.
* NMIX    number of mixtures in the nodal calculation.
* NALB    number of physical albedos.
* EPSOUT  convergence epsilon for the power method.
* MAXOUT  maximum number of iterations for the power method.
* MAT     material mixtures.
* XX      mesh spacings.
* IQFR    boundary condition information.
* QFR     albedo function information.
* DIFF    diffusion coefficients
* SIGR    removal cross sections.
* CHI     fission spectra.
* SIGF    nu times fission cross section.
* SCAT    scattering cross section.
* BETA    albedos.
* FD      discontinuity factors.
* IMPX    edition flag.
*
*Parameters: output
* KEFF    effective multiplication factor.
* SAV     unknown array (neutron fluxes and currents).
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER NUN,NG,LX1,NMIX,NALB,MAXOUT,IMPX,MAT(LX1),IQFR(6,LX1)
      REAL EPSOUT,XX(LX1),QFR(6,LX1),DIFF(NMIX,NG),SIGR(NMIX,NG),
     1 CHI(NMIX,NG),SIGF(NMIX,NG),SCAT(NMIX,NG,NG),BETA(NALB,NG,NG),
     2 FD(NMIX,2,NG,NG),KEFF,SAV(NUN,NG)
*----
*  LOCAL VARIABLES
*----
      REAL(KIND=8) :: DEPS,DKEFF
      REAL :: COEF(2),KEFF_OLD
*----
*  ALLOCATABLE ARRAYS
*----
      COMPLEX(KIND=8), ALLOCATABLE, DIMENSION(:,:,:) :: L,R
      REAL, ALLOCATABLE, DIMENSION(:) :: XXX,WORK1,WORK2,WORK4,WORK5
      REAL, ALLOCATABLE, DIMENSION(:,:) :: WORK3
      REAL, ALLOCATABLE, DIMENSION(:,:,:) :: QFR2,DRIFT,AGGX
      REAL(KIND=8), ALLOCATABLE, DIMENSION(:) :: FUNKN
      REAL(KIND=8), ALLOCATABLE, DIMENSION(:,:) :: A
*
      ALB(X)=0.5*(1.0-X)/(1.0+X)
*----
*  SCRATCH STORAGE ALLOCATION
*----
      N=LX1*NG
      ALLOCATE(QFR2(6,LX1,NG))
      ALLOCATE(L(NG,2*NG,LX1),R(NG,2*NG,LX1))
      ALLOCATE(XXX(LX1+1),DRIFT(2,LX1,NG),WORK1(NG),WORK2(NG),
     1 WORK3(NG,NG),WORK4(NG),WORK5(NG))
      ALLOCATE(AGGX(LX1,LX1,NG),A(N,2*N),FUNKN(N))
*----
*  ALBEDO PROCESSING
*----
      QFR2(:6,:LX1,:NG)=0.0
      DO IG=1,NG
        DO IQW=1,2
          DO IEL=1,LX1
            IALB=IQFR(IQW,IEL)
            IF(IALB > 0) THEN
              IF(IALB.GT.NALB) CALL XABORT('NSSFL3: BETA OVERFLOW.')
              QFR2(IQW,IEL,IG)=QFR(IQW,IEL)*ALB(BETA(IALB,IG,IG))
            ELSE
              QFR2(IQW,IEL,IG)=QFR(IQW,IEL)
            ENDIF
          ENDDO
        ENDDO
      ENDDO
*
      DRIFT(:2,:LX1,:NG)=0.0
      FUNKN(:N)=1.
      KEFF=1.E0
      KEFF_OLD=0.
      XXX(1)=0.
      DO IEL=1,LX1
        XXX(IEL+1)=XXX(IEL)+XX(IEL)
      ENDDO
*----
*  NODAL CORRECTION LOOP
*----
      JTER=0
      DO WHILE(ABS(KEFF_OLD-KEFF) >= EPSOUT)
        JTER=JTER+1
        IF(IMPX.GT.0) THEN
          WRITE(6,'(36H NSSFL3: Nodal correction iteration=,I5)')
     1    JTER
        ENDIF
        IF(JTER > MAXOUT) CALL XABORT('NSSFL3: NODAL ITERATION FAILURE')
        !
        ! set coarse mesh finite difference matrix
        A(:N,:2*N)=0.D0
        DO IG=1,NG
          CALL NSSMXX(LX1,NMIX,MAT,XX,IQFR,QFR2(1,1,IG),DIFF(1,IG),
     1    DRIFT(1,1,IG),SIGR(1,IG),AGGX(1,1,IG))
        ENDDO
        IOF=0
        DO IG=1,NG ! IG <-- JG
          A(IOF+1:IOF+LX1,IOF+1:IOF+LX1)=AGGX(:,:,IG)
          JOF=0
          DO JG=1,NG
            DO IEL=1,LX1
              IBM=MAT(IEL)
              IF(IG /= JG) A(IOF+IEL,JOF+IEL)=-XX(IEL)*SCAT(IBM,IG,JG)
              A(IOF+IEL,N+JOF+IEL)=XX(IEL)*CHI(IBM,IG)*SIGF(IBM,JG)
            ENDDO
            JOF=JOF+LX1
          ENDDO
          IOF=IOF+LX1
        ENDDO
        CALL ALSBD(N,N,A,IER,N)
        IF(IER /= 0) CALL XABORT('NSSFL3: SINGULAR MATRIX.')
        !
        ! CMFD power iteration (use double precision)
        KEFF_OLD=KEFF
        DEPS=EPSOUT
        CALL AL1EIGD(N,A(1,N+1),DEPS,MAXOUT,ITER,FUNKN,DKEFF,IMPX)
        KEFF=REAL(DKEFF)
        IF(IMPX > 0) WRITE(6,10) JTER,KEFF,ITER
        IF(IMPX > 1) THEN
          WRITE(6,'(1X,A)') 'NSSFL3: FUNKN='
          IOF=0
          DO IG=1,NG
            WRITE(6,'(1X,1P,14D12.4)') FUNKN(IOF+1:IOF+LX1)
            IOF=IOF+LX1
          ENDDO
        ENDIF
        !
        ! begin construct SAV
        IF(NUN /= 4*LX1+1) CALL XABORT('NSSFL3: INVALID VALUE OF NUN.')
        SAV(:NUN,:NG)=0.D0
        DO IEL=1,LX1
          DO IG=1,NG
            SAV(IEL,IG)=REAL(FUNKN((IG-1)*LX1+IEL))
          ENDDO
        ENDDO
        !
        ! compute nodal coefficients
        DO IEL=1,LX1
          IBM=MAT(IEL)
          IF(IBM == 0) CYCLE
          WORK1(:NG)=DIFF(IBM,:NG)
          WORK2(:NG)=SIGR(IBM,:NG)
          WORK3(:NG,:NG)=SCAT(IBM,:NG,:NG)
          WORK4(:NG)=CHI(IBM,:NG)
          WORK5(:NG)=SIGF(IBM,:NG)
          CALL NSSLR(KEFF,NG,XXX(IEL),XXX(IEL+1),WORK1,WORK2,WORK3,
     1    WORK4,WORK5,L(1,1,IEL),R(1,1,IEL))
        ENDDO
        !
        ! one- and two-node anm relations
        CALL NSSANM(LX1,NG,NMIX,IQFR,QFR2,MAT,FD,L,R,SAV)
        !
        ! compute drift coefficients
        DO IG=1,NG
          DO IEL=1,LX1
            IBM=MAT(IEL)
            IF(IBM == 0) CYCLE
            CALL NSSCO(LX1,NMIX,IEL,MAT,XX,IQFR(1,IEL),QFR2(1,IEL,IG),
     1      DIFF(1,IG),COEF)
            IF(IEL == 1) THEN
              DRIFT(1,IEL,IG)=-(SAV(3*LX1+IEL,IG)+REAL(COEF(1))*
     1        SAV(IEL,IG))/SAV(IEL,IG)
              DRIFT(2,IEL,IG)=-(SAV(3*LX1+IEL+1,IG)+REAL(COEF(2))*
     1        (SAV(IEL+1,IG)-SAV(IEL,IG)))/(SAV(IEL+1,IG)+SAV(IEL,IG))
            ELSE IF(IEL < LX1) THEN
              DRIFT(1,IEL,IG)=-(SAV(3*LX1+IEL,IG)+REAL(COEF(1))*
     1        (SAV(IEL,IG)-SAV(IEL-1,IG)))/(SAV(IEL,IG)+SAV(IEL-1,IG))
              DRIFT(2,IEL,IG)=-(SAV(3*LX1+IEL+1,IG)+REAL(COEF(2))*
     1        (SAV(IEL+1,IG)-SAV(IEL,IG)))/(SAV(IEL+1,IG)+SAV(IEL,IG))
            ELSE
              DRIFT(1,IEL,IG)=-(SAV(3*LX1+IEL,IG)+REAL(COEF(1))*
     1        (SAV(IEL,IG)-SAV(IEL-1,IG)))/(SAV(IEL,IG)+SAV(IEL-1,IG))
              DRIFT(2,IEL,IG)=-(SAV(3*LX1+IEL+1,IG)-REAL(COEF(2))*
     1        SAV(IEL,IG))/SAV(IEL,IG)
            ENDIF
          ENDDO
        ENDDO
      ENDDO
      IF(IMPX.GT.0) WRITE(6,20) KEFF,JTER
*----
*  SCRATCH STORAGE DEALLOCATION
*----
      DEALLOCATE(FUNKN,A,AGGX)
      DEALLOCATE(WORK5,WORK4,WORK3,WORK2,WORK1,DRIFT,XXX)
      DEALLOCATE(R,L,QFR2)
      RETURN
*
   10 FORMAT(14H NSSFL3: JTER=,I4,11H CMFD KEFF=,1P E13.5,
     1 12H OBTAINED IN,I4,12H ITERATIONS.)
   20 FORMAT(18H NSSFL3: ANM KEFF=,F11.8,12H OBTAINED IN,I5,
     1 11H ITERATIONS)
      END
